package CH.psi.midas;

import java.io.*;
import java.util.*;

import CH.psi.midas.*;

public class OdbKey extends java.lang.Object {
  
  public static final int TID_BYTE    =  1; /* unsigned byte         0       255    */
  public static final int TID_SBYTE   =  2; /* signed byte         -128      127    */
  public static final int TID_CHAR    =  3; /* single character      0       255    */
  public static final int TID_WORD    =  4; /* two bytes             0      65535   */
  public static final int TID_SHORT   =  5; /* signed word        -32768    32767   */
  public static final int TID_DWORD   =  6; /* four bytes            0      2^32-1  */
  public static final int TID_INT     =  7; /* signed dword        -2^31    2^31-1  */
  public static final int TID_BOOL    =  8; /* four bytes bool       0        1     */
  public static final int TID_FLOAT   =  9; /* 4 Byte float format                  */
  public static final int TID_DOUBLE  = 10; /* 8 Byte float format                  */
  public static final int TID_BITFIELD= 11; /* 32 Bits Bitfield      0  111... (32) */
  public static final int TID_STRING  = 12; /* zero terminated string               */
  public static final int TID_ARRAY   = 13; /* array with unknown contents          */
  public static final int TID_STRUCT  = 14; /* structure with fixed length          */
  public static final int TID_KEY     = 15; /* key in online database               */
  public static final int TID_LINK    = 16; /* link in online database              */
  public static final int TID_LAST    = 17; /* end of TID list indicator            */

  private int hDB;
  private int hKey;
  private boolean valid;
  private boolean dataValid;
  private boolean hotLinked;
  private String path;
  private String name;
  private int type;
  private int numValues;
  private int itemSize;
  private String data;
  private String array[];
  
  private Experiment experiment;
  private OdbKeyUpdate keyUpdate;
  
  public OdbKey() {
    valid = false;
  }
  
  public OdbKey(Experiment experiment, OdbKey rootKey, String path) throws IOException, OdbKeyNotFoundException {

    // check if valid path
    if (path.equals(""))
      throw new IOException("Key path not set");
      
    this.experiment = experiment;
    hDB = experiment.getHdb();
    valid = false;
    dataValid = false;
    hotLinked = false;
    this.path = rootKey.path + "/" + path;
    
    String r = experiment.rpcCall("db_find_key&"+hDB+"&"+rootKey.hKey+"&"+path);
    if (experiment.replyParam(r, 0).equals("1")) {
      valid = true;
      hKey = Integer.parseInt(experiment.replyParam(r, 1));
      
      getInfo();
    } else throw new OdbKeyNotFoundException("OdbKey "+rootKey+"/"+path);
  }

  public OdbKey(Experiment experiment, String path) throws IOException, OdbKeyNotFoundException {

    // check if valid path
    if (path.equals(""))
      throw new IOException("Key path not set");

    this.experiment = experiment;
    hDB = experiment.getHdb();
    valid = false;
    dataValid = false;
    hotLinked = false;
    this.path = path;
    
    String r = experiment.rpcCall("db_find_key&"+hDB+"&0&"+path);
    if (experiment.replyParam(r, 0).equals("1")) {
      valid = true;
      hKey = Integer.parseInt(experiment.replyParam(r, 1));
      
      getInfo();
    } else throw new OdbKeyNotFoundException("OdbKey "+path);
  }

  public OdbKey(Experiment experiment, String path, int type) throws IOException, OdbKeyNotFoundException {

    // check if valid path
    if (path.equals(""))
      throw new IOException("Key path not set");

    this.experiment = experiment;
    hDB = experiment.getHdb();
    valid = false;
    dataValid = false;
    hotLinked = false;
    this.path = path;
    
    // create key with given type
    experiment.rpcCall("db_create_key&"+hDB+"&0&"+path+"&"+type);

    String r = experiment.rpcCall("db_find_key&"+hDB+"&0&"+path);
    if (experiment.replyParam(r, 0).equals("1")) {
      valid = true;
      hKey = Integer.parseInt(experiment.replyParam(r, 1));
      
      getInfo();
    } else throw new OdbKeyNotFoundException("OdbKey "+path);
  }

  public OdbKey(Experiment experiment, int hKey) throws IOException, OdbKeyNotFoundException {

    // check if valid path
    if (hKey == 0)
      throw new OdbKeyNotFoundException("Invalid hKey=0");

    this.experiment = experiment;
    this.hDB = experiment.getHdb();
    this.hKey = hKey;

    // retrieve full path
    String r = experiment.rpcCall("db_get_path&"+hDB+"&"+hKey+"&256");
    this.path = experiment.replyParam(r, 1);
    
    valid = true;
    dataValid = false;
    hotLinked = false;
      
    getInfo();
  }

  // optimized version used by enumSubKey to avoid retrieval of path
  private OdbKey(Experiment experiment, int hKey, OdbKey parent) throws 
            IOException, OdbKeyNotFoundException {

    // check if valid path
    if (hKey == 0)
      throw new OdbKeyNotFoundException("Invalid hKey=0");

    this.experiment = experiment;
    this.hDB = experiment.getHdb();
    this.hKey = hKey;
    valid = true;
    dataValid = false;
    hotLinked = false;
    
    getInfo();

    // construct full path
    this.path = parent.getName() + "/" + name;
  }
  
  public int getHkey() {
    return hKey;
  }
  
  public String getPath() {
    return path;
  }

  public int getType() {
    return type;
  }

  public int getNumValues() {
    return numValues;
  }

  public String getName()
  {
    return this.name;
  }
  
  public void getInfo() throws IOException {
    String r = experiment.rpcCall("db_get_key_info&"+hDB+"&"+hKey+"&32");
    if (experiment.replyParam(r, 0).equals("1")) {
      this.name = experiment.replyParam(r, 1);
      this.type = Integer.parseInt(experiment.replyParam(r, 2));
      this.numValues = Integer.parseInt(experiment.replyParam(r, 3));
      this.itemSize = Integer.parseInt(experiment.replyParam(r, 4));
    }
  }

  public void getData() throws IOException {
    
    if (type == TID_KEY) // key doesn't contain data
      data = null;
    else {
      String r = experiment.rpcCall("db_get_data1&"+hDB+"&"+hKey+"&64000&"+type);
      if (experiment.replyParam(r, 0).equals("1")) {
        this.numValues = Integer.parseInt(experiment.replyParam(r, 1));
        if (this.numValues == 1)
          this.data = experiment.replyParam(r, 2);
        else {
          this.array = new String[numValues];        
          for (int i=0 ; i<numValues ; i++)
            this.array[i] = experiment.replyParam(r, 2+i);
        }
      }
    }
    
    dataValid = true;
  }
 
  public void getDataAt(int index) throws IOException {
    
    if (type == TID_KEY) // key doesn't contain data
      data = null;
    else {
      String r = experiment.rpcCall("db_get_data_index&"+hDB+"&"+hKey+"&64000&"+index+"&"+type);
      if (experiment.replyParam(r, 0).equals("1")) {
        if (numValues == 1)
          this.data = experiment.replyParam(r, 2);
        else
          this.array[index] = experiment.replyParam(r, 2);
      }
    }
  }

  public void setText(String str) throws IOException, OdbNoAccessException {
    if (numValues > 1) {

      // break string apart into individual values
      int i, n;
      
      StringTokenizer st = new StringTokenizer(str, "[], ");
      n = st.countTokens();
      String tmp[] = new String[n];
      
      String rpc = "db_set_data&"+hDB+"&"+hKey+"&"+n+" "+type;
      
      for (i=0 ; i<n ; i++) {
        tmp[i] = st.nextToken();
        rpc += "&"+tmp[i];
      }
      rpc += "&0&"+n+"&"+type;
      String r = experiment.rpcCall(rpc);
      if (experiment.replyParam(r, 0).equals("318"))
        throw new OdbNoAccessException("OdbKey "+this.path);
        
      // if successful, copy data to this  
      array = new String[n];  
      for (i=0 ; i<n ; i++)
        array[i] = tmp[i];
      
    } else {
      
      // send individual value
      String r = experiment.rpcCall("db_set_data&"+hDB+"&"+hKey+
                                    "&1 "+type+"&"+str+"&0&1&"+type);
      if (experiment.replyParam(r, 0).equals("318"))
        throw new OdbNoAccessException("OdbKey "+this.path);
      this.data = str;  
    }
  }
  
  public void setTextAt(String str, int index) throws 
                IOException,ArrayIndexOutOfBoundsException,
                OdbNoAccessException {
    if (numValues == 1)
      setText(str);
    else {
      if (index < numValues) {
        String r = experiment.rpcCall("db_set_data_index&"+hDB+"&"+hKey+
                                      "&1 "+type+"&"+str+"&0&"+index+"&"+type);
        if (experiment.replyParam(r, 0).equals("318"))
          throw new OdbNoAccessException("OdbKey "+this.path);
        array[index] = str;
      } else throw new IndexOutOfBoundsException("OdbKey " + this.path);
    }
  }

  public void setTextAt(String str, int index, int size) throws 
                IOException,ArrayIndexOutOfBoundsException,
                OdbNoAccessException {

    String r = experiment.rpcCall("db_set_data_index&"+hDB+"&"+hKey+
                                  "&1 "+type+"&"+str+"&"+size+"&"+index+"&"+type);
    if (experiment.replyParam(r, 0).equals("318"))
      throw new OdbNoAccessException("OdbKey "+this.path);
    getData();
  }

  public void setTextAt(String str, int index, boolean notify) throws 
                IOException,ArrayIndexOutOfBoundsException,
                OdbNoAccessException {
    if (index < numValues) {
      String r = experiment.rpcCall("db_set_data_index2&"+hDB+"&"+hKey+
                                    "&1 "+type+"&"+str+"&0&"+index+"&"+type+(notify?"&1":"&0"));
      if (experiment.replyParam(r, 0).equals("318"))
        throw new OdbNoAccessException("OdbKey "+this.path);
      array[index] = str;
    } else throw new IndexOutOfBoundsException("OdbKey " + this);
  }

  public void setNumValues(int numValues) throws IOException, OdbNoAccessException {

    String r = experiment.rpcCall("db_set_num_values&"+hDB+"&"+hKey+"&"+numValues);
    if (experiment.replyParam(r, 0).equals("318"))
      throw new OdbNoAccessException("OdbKey "+this.path);
    getData();
  }
  
  public void hotLink(OdbKeyUpdate ku) throws IOException, TooManyListenersException {

    // get inital data
    if (!dataValid)
      getData();
      
    String r = experiment.rpcCall("db_add_open_record&"+hDB+"&"+hKey+"&0");
    if (!experiment.replyParam(r, 0).equals("1"))
      throw new TooManyListenersException("OdbKey "+path);
      
    // register hotlink in experiment
    experiment.addHotLink(this);
    
    // save update function
    keyUpdate = ku;
    
    hotLinked = true;
  }

  public void hotLink() throws IOException, TooManyListenersException {
    hotLink(null);
  }

  public void hotUnlink() throws IOException {
    if (hotLinked) {
      experiment.removeHotLink(this);
      experiment.rpcCall("db_remove_open_record&"+hDB+"&"+hKey);
      
      hotLinked = false;
    }
  }

  public void dispatchUpdate() throws IOException {
    getData();
    if (keyUpdate != null)
      keyUpdate.update(this);
  }
  
  public String toString() {
    
    // if data not yet fetched, do now
    if (!dataValid)
      try {
        getData();
      } catch (IOException e) { System.out.println(e); }
  
    // return name instead data for keys
    if (type == TID_KEY)
      return name;
      
    // return single value  
    if (numValues == 1)
      return data;
    else {
      // return comma separated values
      String str = "[";
      int i;
      
      for (i=0 ; i<numValues ; i++) {
        str += array[i];
        if (i<numValues-1)
          str += ',';
      }
      
      str += "]";
      
      return str;
    }
  }

  public String toString(int i) {
    
    // if data not yet fetched, do now
    if (!dataValid)
      try {
        getData();
      } catch (IOException e) { System.out.println(e); }
  
    // return name instead data for keys
    if (type == TID_KEY)
      return name;
      
    // return single value  
    if (numValues == 1)
      return data;
    else
      return array[i];
  }

  public OdbKey enumSubKey(int index) throws OdbKeyNotFoundException, IOException {
    String r = experiment.rpcCall("db_enum_key&"+hDB+"&"+this.hKey+"&"+index);
    if (experiment.replyParam(r, 0).equals("1")) {
      int hSubkey = Integer.parseInt(experiment.replyParam(r, 1));
      OdbKey subkey = new OdbKey(experiment, hSubkey, this);
      return subkey;
    } else return null;
  }

  public void deleteKey() throws IOException {
    experiment.removeHotLink(this);
    experiment.rpcCall("db_delete_key&"+hDB+"&"+this.hKey+"&0");
  }
  
  protected void finalize() throws Exception {
    experiment.removeHotLink(this);
  }
} 