// -*- C++ -*-
//
// FBmodule.h
// Header file
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Apr 16 18:09:35 JST 2002
//
// $Id: FBcrate.h,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// Revision History
//
// $Log: FBcrate.h,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#ifndef FBMODULE_H_INCLUDED
#define FBMODULE_H_INCLUDED

#include <string>
#include <list>

//
class FBcrate;

//
class FBslot {
public:
  FBslot( FBcrate* crate,
          int pa, 
          const std::string& name,
          const std::list<std::string>& init,
          const std::list<int>& channels );
  
  ~FBslot();
  //
  bool  init();
  //
  bool                   valid() const { return m_valid; }
  int                    pa()   const { return m_pa; }
  const std::string&     name() const { return m_name; }
  const std::list<std::string>&  commandList() const { return m_commandlist; }
  const std::list<int>&  channelList() const { return m_channellist; }
private:
  //
  FBcrate*                m_crate;
  int                     m_pa;
  std::string             m_name;
  std::list<std::string>  m_commandlist;
  std::list<int>          m_channellist;
  bool                    m_valid;
  //
};


//
class FBcrate {
public:
  FBcrate();
  ~FBcrate();
  void setFunc( 
               void (*frc)(unsigned int pa,unsigned int sa,unsigned int* data),
               void (*fwc)(unsigned int pa,unsigned int sa,unsigned int  data),
               void (*frd)(unsigned int pa,unsigned int sa,unsigned int* data),
               void (*fwd)(unsigned int pa,unsigned int sa,unsigned int  data),
               void (*reset)()
               );
  //
  inline void  frc( unsigned int pa,unsigned int sa,unsigned int* data );
  inline void  fwc( unsigned int pa,unsigned int sa,unsigned int  data );
  inline void  frd( unsigned int pa,unsigned int sa,unsigned int* data );
  inline void  fwd( unsigned int pa,unsigned int sa,unsigned int  data );
  inline void  reset();
  
  //
  bool  init();
  int   readConf( const std::string& filename );
  void  dumpConf();
  const std::list<FBslot>&  slotList() const { return m_slot; }
  
private:
  std::list<FBslot>     m_slot;

  void (*m_frc)( unsigned int pa,unsigned int sa,unsigned int* data );
  void (*m_fwc)( unsigned int pa,unsigned int sa,unsigned int  data );
  void (*m_frd)( unsigned int pa,unsigned int sa,unsigned int* data );
  void (*m_fwd)( unsigned int pa,unsigned int sa,unsigned int  data );
  void (*m_reset)();
};

// inline
inline void
FBcrate::frc( unsigned int pa,unsigned int sa,unsigned int* data )
{
  m_frc( pa,sa,data );
}

inline void
FBcrate::fwc( unsigned int pa,unsigned int sa,unsigned int  data )
{
  m_fwc( pa,sa,data );
}

inline void
FBcrate::frd( unsigned int pa,unsigned int sa,unsigned int* data )
{
  m_frd( pa,sa,data );
}

inline void
FBcrate::fwd( unsigned int pa,unsigned int sa,unsigned int  data )
{
  m_fwd( pa,sa,data );
}

inline void
FBcrate::reset()
{
  m_reset();
}

//
inline void
FBcrate::setFunc(
             void (*frc)(unsigned int pa,unsigned int sa,unsigned int* data ),
             void (*fwc)(unsigned int pa,unsigned int sa,unsigned int  data ),
             void (*frd)(unsigned int pa,unsigned int sa,unsigned int* data ),
             void (*fwd)(unsigned int pa,unsigned int sa,unsigned int  data ),
             void (*reset)() )
{
  m_frc   = frc;
  m_fwc   = fwc;
  m_frd   = frd;
  m_fwd   = fwd;
  m_reset = reset;
}


//
#endif // FBMODULE_H_INCLUDED

