// -*- C++ -*-
//
// GsimGeneral.h
// Common Definition
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Fri Mar 22 12:17:34 JST 2002
//
// $Id: GsimGeneral.h,v 1.1.1.1 2002/08/13 12:48:09 kensh Exp $
//
// Revision History
//
// $Log: GsimGeneral.h,v $
// Revision 1.1.1.1  2002/08/13 12:48:09  kensh
//
//
// Start version.
//
//

#ifndef GSIMGENERAL_H_INCLUDED
#define GSIMGENERAL_H_INCLUDED

//
#include <string>
#include "CLHEP/Vector/ThreeVector.h"
#include "TupleManager/TMNtuple.h"



////////////////
// User must provide these functions.
void  GsimFFuser();
void  GsimUserGeom();
void  GsimUserMate();
void  Gukine();
void  Guout();
void  Gustep();
void  Gutrak();
void  Ugeom();
void  Uginit();
void  Uglast();


////////////////
// 
#define  MAX_MECH  10000


////////////////
// Global variables

// GsimDetView.cc
extern int    isGxsim;
extern int    disp_2buf;
extern float  disp_zoom[];
extern float  disp_center[];
extern float  disp_rotth;
extern float  disp_rotph;
extern float  disp_rotps;

// GsimMain.cc
extern int        Display;
extern int        SaveMode;
extern TMNtuple*  Ituple;
extern TMNtuple*  Otuple;
extern TMNtuple*  Ogenpart;
extern TMNtuple*  Ogsimrand;

// GsimMate.cc
extern float  mfstep;

// GsimParticleGun.cc
extern float  TargetZ;


////////////////
// General functions provided by libgsim-general

// GsimDetView.cc
void  GsimDetView( int iview );

// GsimEvent.cc
int   GsimEvent( int loop );
int   GsimEventSkip( int skip );

// GsimGenParticle.cc
int   GsimGenParticle();

// GsimFFread.cc
void  GsimFFread();

// GsimMain.cc
void  GsimUsage();
int   GsimParams( int argc, const char* argv[] );
int   GsimEventLoop();

// GsimParticleGun.cc
void  GsimParticleGun( int pid, 
                       double pmin,  double pmax,
                       double pthmin,double pthmax, 
                       double pphmin,double pphmax,
                       double rmin,  double rmax,
                       double phmin, double phmax,
                       double zmin,  double zmax,
                       Hep3Vector& p, Hep3Vector& vtx );
void  GsimParticleGunRTP( int pid, 
                          double pmin,  double pmax,
                          double pthmin,double pthmax, 
                          double pphmin,double pphmax,
                          double rmin,  double rmax,
                          double phmin, double phmax,
                          double zmin,  double zmax,
                          Hep3Vector& p, Hep3Vector& vtx );
void  GsimKlongDecayMode( int mode );
void  GsimGenKlongCol6( double z,int absorber, Hep3Vector& p,Hep3Vector& vtx );
void  GsimGenKlongTarget( double z, Hep3Vector& p, Hep3Vector& vtx );


// gsim_psfile.F
extern "C" {
  void  gsim_open_epsfile_( int*,const char*, int );
  void  gsim_open_psfile_(  int*,const char*, int );
  void  gsim_close_psfile_( int* );
}


////////////////
// inline

//
inline double
deg2rad( double deg ){ return( deg/180.*M_PI ); }

//
inline double
rad2deg( double rad ){ return( rad/M_PI*180. ); }

//
inline void
OPEN_PSFILE( int lun, std::string fname )
{
  if ( fname.find( ".eps" ) < fname.length() ){
    gsim_open_epsfile_( &lun, fname.c_str(), fname.length() );
  }
  else{
    gsim_open_psfile_( &lun, fname.c_str(), fname.length() );
  }
}

//
inline void
CLOSE_PSFILE( int lun )
{
  gsim_close_psfile_( &lun );
}


#endif // GSIMGENERAL_H_INCLUDED
