// -*- C++ -*-
//
// MidasFileIO.cc
// FileIO class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Jul 10 18:40:57 JST 2001
//
// $Id: MidasBank.cc,v 1.2 2002/08/29 13:25:34 yamaga Exp $
//
// Revision History
//
// $Log: MidasBank.cc,v $
// Revision 1.2  2002/08/29 13:25:34  yamaga
// Many functions added and modified...
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <list>
#include <string>
#include <string.h>
#include "midas.h"
#include "MidasManager/MidasBank.h"


//////
#define DEBUG  0


//////
MidasBank::MidasBank( int bufsize )
  : m_bufsize( bufsize )
{
  if (DEBUG) std::cerr << "MidasBank::MidasBank" << std::endl;
  m_unalignedbuf = new char[ m_bufsize+16 ];
}


//////
MidasBank::MidasBank( const MidasBank& mb )
  : m_bufsize( mb.m_bufsize )
{
  if (DEBUG) std::cerr << "MidasBank::MidasBank copy" << std::endl;

  m_unalignedbuf = new char[ mb.m_bufsize+16 ];
  memcpy( (char*)ALIGN((PTYPE)m_unalignedbuf),
          (char*)ALIGN((PTYPE)mb.m_unalignedbuf),
          m_bufsize );
}


//////
MidasBank::~MidasBank()
{
  if (DEBUG) std::cerr << "MidasBank::~MidasBank" << std::endl;

  delete m_unalignedbuf;
}


//////
MidasBank&
MidasBank::operator=( const MidasBank& mb )
{
  if (DEBUG) std::cerr << "MidasBank::operator=" << std::endl;

  delete m_unalignedbuf;
  m_unalignedbuf = new char[ mb.m_bufsize+16 ];
  memcpy( (char*)ALIGN((PTYPE)m_unalignedbuf), 
          (char*)ALIGN((PTYPE)mb.m_unalignedbuf),
          m_bufsize );
  return( *this );
}


//////
bool
MidasBank::realloc( int size )
{
  if (DEBUG)
    std::cerr << "MidasBank::realloc :"
              << " size= " << size
              << " m_buf= " << m_bufsize
              << std::endl;

  if ( size > m_bufsize ){
    m_bufsize = size;
    delete  m_unalignedbuf;
    m_unalignedbuf = new char[ m_bufsize+16 ];
    if ( m_unalignedbuf == NULL )
      return( false );
  }
  return( true );
}


//////
void
MidasBank::init( int event_id, int mask, int serialnumber )
{
  if (DEBUG) std::cerr << "MidasBank::init" << std::endl;

  bm_compose_event( eventHeader(), event_id, mask, 0, serialnumber );
  bk_init32( eventHeader()+1 );
}


//////
void
MidasBank::initFilter( const EVENT_HEADER* pheader )
{
  if (DEBUG) std::cerr << "MidasBank::initFilter" << std::endl;

  bm_compose_event( eventHeader(), 
                    pheader->event_id,
                    pheader->trigger_mask,
                    0, 
                    pheader->serial_number );
  eventHeader()->time_stamp = pheader->time_stamp;

  bk_init32( eventHeader()+1 );
}


//////
std::list<BANK32>
MidasBank::getBankDefs()
{
  if (DEBUG) std::cerr << "MidasBank::getBankDefs" << std::endl;

  std::list<BANK32>  banklist;

  // loop over banks
  char*    pbuf = NULL;
  BANK32*  pbk = NULL;
  while( true ){
    bk_iterate32( eventBuffer(), &pbk, &pbuf );
    if ( pbk == NULL )
      break; // no more bank
    banklist.push_back( *pbk );
  }

  return( banklist );
}


//////
int
MidasBank::getBank( const std::string& name, void* pdata )
{
  if (DEBUG) std::cerr << "MidasBank::getBank" << std::endl;

  // look for the bank
  return( bk_locate( eventBuffer(), (char*)name.c_str(), pdata ) );
}


//////
void*
MidasBank::createBank( const std::string& name, int TID_type, int n_elements )
{
  if (DEBUG) std::cerr << "MidasBank::createBank" << std::endl;
  
  char*  pbuf;
  bk_create( eventBuffer(), (char*)name.c_str(), TID_type, &pbuf );
  return( pbuf );
}


//////
int
MidasBank::closeBank( void* pdata )
{
  if (DEBUG) std::cerr << "MidasBank::closeBank" << std::endl;
  
  return( bk_close( eventBuffer(), pdata ) );
}


//////
int
MidasBank::copyBank( const MidasBank* bank, const std::string& name )
{
  if (DEBUG) std::cerr << "MidasBank::copyBank" << std::endl;
  
  // search bank
  char*    pbuf = NULL;
  BANK32*  pbk = NULL;
  while( true ){
    bk_iterate32( (void*)bank->eventBuffer(), &pbk, &pbuf );
    if ( pbk == NULL )
      break; // no more bank
    //
    if ( strncmp( pbk->name,name.c_str(),4 ) == 0 ){
      // copy
      char*  pdata;
      bk_create( eventBuffer(), (char*)name.c_str(), pbk->type, &pdata );
      memcpy( pdata, pbuf, pbk->data_size );
      pdata += pbk->data_size;
      return( bk_close( eventBuffer(), pdata ) );
    }
  }
  
  return( 0 );
}


//////
int
MidasBank::copyAllBank( const MidasBank* bank )
{
  if (DEBUG) std::cerr << "MidasBank::copyBank" << std::endl;
  
  // search bank
  char*    pbuf = NULL;
  BANK32*  pbk = NULL;
  while( true ){
    bk_iterate32( (void*)bank->eventBuffer(), &pbk, &pbuf );
    if ( pbk == NULL )
      break; // no more bank
    // copy
    char*  pdata;
    std::string  name( pbk->name,4 );
    bk_create( eventBuffer(), (char*)name.c_str(), pbk->type, &pdata );
    memcpy( pdata, pbuf, pbk->data_size );
    pdata += pbk->data_size;
    bk_close( eventBuffer(), pdata );
  }
  
  return( 0 );
}


//////
int
MidasBank::addBank( const std::string& name,int TID_type, 
                    int n_elements, void* pdata )
{
  if (DEBUG) std::cerr << "MidasBank::addBank" << std::endl;

  // 
  char*  pbuf;
  int    bytes = TID_type * n_elements;
  bk_create( eventBuffer(), (char*)name.c_str(), TID_type, &pbuf );
  memcpy( pbuf, pdata, bytes );
  pbuf += bytes;
  int  ret = bk_close( eventBuffer(), pbuf );
  return( ret );
}


//////
void
MidasBank::showBankDefs()
{
  static char* tname[] = {
    "?","BYTE","SBYTE","CHAR","WORD",
    "SHORT","DWORD","INT","BOOL","FLOAT",
    "DOUBLE","BITFIELD","STRING","ARRAY","STRUCT",
    "KEY","LINK","LAST",
  };
  static int  tsize[] = {
    1,1,1,1,2,
    2,4,4,4,4,
    8,4,1,1,1,
    1,1,1,
  };
  
  std::list<BANK32>  blist = getBankDefs();
  for( std::list<BANK32>::iterator i=blist.begin();
       i!=blist.end(); i++ ){
    std::cout << ""
              << "Bank " 
              << std::string( i->name,4 )
              << " type=" 
              << ( (0<i->type && i->type<18) ? tname[i->type] : "?" )
              << " Ndata="
              << ( (0<i->type && i->type<18) ? 
                   i->data_size / tsize[i->type] : i->data_size )
              << " size="
              << i->data_size
              << " (0x" << std::hex << i->data_size << ")" << std::dec
              << std::endl;
  }
}
